# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
from enum import Enum


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Lightsail2(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field src_ip_addr: ax25_frame.payload.ax25_info.src_ip_addr
    :field dst_ip_addr: ax25_frame.payload.ax25_info.dst_ip_addr
    :field src_port: ax25_frame.payload.ax25_info.body.src_port
    :field dst_port: ax25_frame.payload.ax25_info.body.dst_port
    :field type: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.type
    :field daughter_atmp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.daughter_atmp
    :field daughter_btmp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.daughter_btmp
    :field threev_pltmp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.threev_pltmp
    :field rf_amptmp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.rf_amptmp
    :field nx_tmp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.nx_tmp
    :field px_tmp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.px_tmp
    :field ny_tmp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.ny_tmp
    :field py_tmp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.py_tmp
    :field nz_tmp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.nz_tmp
    :field pz_tmp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.pz_tmp
    :field atmelpwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.atmelpwrcurr
    :field atmelpwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.atmelpwrbusv
    :field threev_pwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.threev_pwrcurr
    :field threev_pwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.threev_pwrbusv
    :field threev_plpwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.threev_plpwrcurr
    :field threev_plpwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.threev_plpwrbusv
    :field fivev_plpwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.fivev_plpwrcurr
    :field fivev_plpwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.fivev_plpwrbusv
    :field daughter_apwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.daughter_apwrcurr
    :field daughter_apwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.daughter_apwrbusv
    :field daughter_bpwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.daughter_bpwrcurr
    :field daughter_bpwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.daughter_bpwrbusv
    :field nx_intpwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.nx_intpwrcurr
    :field nx_intpwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.nx_intpwrbusv
    :field nx_extpwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.nx_extpwrcurr
    :field nx_extpwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.nx_extpwrbusv
    :field px_intpwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.px_intpwrcurr
    :field px_intpwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.px_intpwrbusv
    :field px_extpwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.px_extpwrcurr
    :field px_extpwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.px_extpwrbusv
    :field ny_intpwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.ny_intpwrcurr
    :field ny_intpwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.ny_intpwrbusv
    :field ny_extpwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.ny_extpwrcurr
    :field ny_extpwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.ny_extpwrbusv
    :field py_intpwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.py_intpwrcurr
    :field py_intpwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.py_intpwrbusv
    :field py_extpwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.py_extpwrcurr
    :field py_extpwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.py_extpwrbusv
    :field nz_extpwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.nz_extpwrcurr
    :field nz_extpwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.nz_extpwrbusv
    :field usercputime: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.usercputime
    :field syscputime: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.syscputime
    :field idlecputime: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.idlecputime
    :field processes: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.processes
    :field memfree: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.memfree
    :field buffers: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.buffers
    :field cached: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.cached
    :field datafree: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.datafree
    :field nanderasures: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.nanderasures
    :field beaconcnt: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.beaconcnt
    :field time: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.time
    :field boottime: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.boottime
    :field long_dur_counter: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sys.long_dur_counter
    :field batt_pwr_draw: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.battPwrDraw
    :field adcs_mode: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.adcs_mode
    :field flags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.flags
    :field q0_act: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.q0_act
    :field q1_act: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.q1_act
    :field q2_act: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.q2_act
    :field q3_act: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.q3_act
    :field x_rate: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.x_rate
    :field y_rate: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.y_rate
    :field z_rate: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.z_rate
    :field gyro_px: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.gyro_pxy.val_a
    :field gyro_py: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.gyro_pxy.val_b
    :field gyro_iz: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.gyro_piz.val_a
    :field gyro_pz: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.gyro_piz.val_b
    :field gyro_ix: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.gyro_ixy.val_a
    :field gyro_iy: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.gyro_ixy.val_b
    :field sol_nxx: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sol_nxx
    :field sol_nxy: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sol_nxy
    :field sol_nyx: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sol_nyx
    :field sol_nyy: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sol_nyy
    :field sol_nzx: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sol_nzx
    :field sol_nzy: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sol_nzy
    :field sol_pxx: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sol_pxx
    :field sol_pxy: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sol_pxy
    :field sol_pyx: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sol_pyx
    :field sol_pyy: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.sol_pyy
    :field mag_nxx: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.mag_nxxy.val_a
    :field mag_nxy: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.mag_nxxy.val_b
    :field mag_nxz: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.mag_npxz.val_a
    :field mag_pxz: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.mag_npxz.val_b
    :field mag_pxx: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.mag_pxxy.val_a
    :field mag_pxy: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.mag_pxxy.val_b
    :field mag_nyz: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.mag_npyz.val_a
    :field mag_pyz: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.mag_npyz.val_b
    :field mag_pyx: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.mag_pyxy.val_a
    :field mag_pyy: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.mag_pyxy.val_b
    :field wheel_rpm: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.wheel_rpm
    :field cam0_status: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.cam0.status
    :field cam0_temp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.cam0.temp
    :field cam0_last_contact: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.cam0.last_contact
    :field cam0_pics_remaining: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.cam0.pics_remaining
    :field cam0_retry_fails: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.cam0.retry_fails
    :field cam1_status: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.cam1.status
    :field cam1_temp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.cam1.temp
    :field cam1_last_contact: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.cam1.last_contact
    :field cam1_pics_remaining: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.cam1.pics_remaining
    :field cam1_retry_fails: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.cam1.retry_fails
    :field torqx_pwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.torqx_pwrcurr
    :field torqx_pwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.torqx_pwrbusv
    :field torqy_pwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.torqy_pwrcurr
    :field torqy_pwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.torqy_pwrbusv
    :field torqz_pwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.torqz_pwrcurr
    :field torqz_pwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.torqz_pwrbusv
    :field motor_pwrcurr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.motor_pwrcurr
    :field motor_pwrbusv: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.motor_pwrbusv
    :field pic_panel_flags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.pic_panel_flags
    :field motor_cnt: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.motor_cnt
    :field motor_limit: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.motor_limit
    :field bat0_curr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat0.curr
    :field bat0_volt: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat0.volt
    :field bat0_temp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat0.temp
    :field bat0_flags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat0.flags
    :field bat0_ctlflags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat0.ctlflags
    :field bat1_curr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat1.curr
    :field bat1_volt: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat1.volt
    :field bat1_temp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat1.temp
    :field bat1_flags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat1.flags
    :field bat1_ctlflags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat1.ctlflags
    :field bat2_curr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat2.curr
    :field bat2_volt: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat2.volt
    :field bat2_temp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat2.temp
    :field bat2_flags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat2.flags
    :field bat2_ctlflags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat2.ctlflags
    :field bat3_curr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat3.curr
    :field bat3_volt: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat3.volt
    :field bat3_temp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat3.temp
    :field bat3_flags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat3.flags
    :field bat3_ctlflags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat3.ctlflags
    :field bat4_curr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat4.curr
    :field bat4_volt: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat4.volt
    :field bat4_temp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat4.temp
    :field bat4_flags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat4.flags
    :field bat4_ctlflags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat4.ctlflags
    :field bat5_curr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat5.curr
    :field bat5_volt: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat5.volt
    :field bat5_temp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat5.temp
    :field bat5_flags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat5.flags
    :field bat5_ctlflags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat5.ctlflags
    :field bat6_curr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat6.curr
    :field bat6_volt: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat6.volt
    :field bat6_temp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat6.temp
    :field bat6_flags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat6.flags
    :field bat6_ctlflags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat6.ctlflags
    :field bat7_curr: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat7.curr
    :field bat7_volt: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat7.volt
    :field bat7_temp: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat7.temp
    :field bat7_flags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat7.flags
    :field bat7_ctlflags: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.bat7.ctlflags
    :field comm_rxcount: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.comm.rxcount
    :field comm_txcount: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.comm.txcount
    :field comm_rxbytes: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.comm.rxbytes
    :field comm_txbytes: ax25_frame.payload.ax25_info.body.body.lsb_beacondata.comm.txbytes
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Lightsail2.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Lightsail2.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Lightsail2.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Lightsail2.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Lightsail2.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Lightsail2.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Lightsail2.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Lightsail2.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Lightsail2.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Lightsail2.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Lightsail2.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Lightsail2.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            _on = self.pid
            if _on == 204:
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Lightsail2.Ipv4Pkt(_io__raw_ax25_info, self, self._root)
            elif _on == 240:
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Lightsail2.NoneL3(_io__raw_ax25_info, self, self._root)
            else:
                self.ax25_info = self._io.read_bytes_full()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ascii")


    class LsbBatterydataT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.curr = self._io.read_s1()
            self.volt = self._io.read_u1()
            self.temp = self._io.read_u1()
            self.flags = self._io.read_u1()
            self.ctlflags = self._io.read_u1()


    class Packedsigned2x12T(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.a_high = self._io.read_s1()
            self.b_high = self._io.read_s1()
            self.ab_low = self._io.read_u1()

        @property
        def val_a(self):
            if hasattr(self, '_m_val_a'):
                return self._m_val_a

            self._m_val_a = ((self.a_high << 4) | (self.ab_low & 15))
            return getattr(self, '_m_val_a', None)

        @property
        def val_b(self):
            if hasattr(self, '_m_val_b'):
                return self._m_val_b

            self._m_val_b = ((self.b_high << 4) | (self.ab_low & (240 >> 4)))
            return getattr(self, '_m_val_b', None)


    class UdpPayload(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.lsb_beacondata = Lightsail2.LsbBeacondataT(self._io, self, self._root)


    class Ipv4Options(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.entries = []
            i = 0
            while not self._io.is_eof():
                self.entries.append(Lightsail2.Ipv4Option(self._io, self, self._root))
                i += 1



    class NoNextHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            pass


    class Ipv6Pkt(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.version = self._io.read_bits_int_be(4)
            self.traffic_class = self._io.read_bits_int_be(8)
            self.flow_label = self._io.read_bits_int_be(20)
            self._io.align_to_byte()
            self.payload_length = self._io.read_u2be()
            self.next_header_type = self._io.read_u1()
            self.hop_limit = self._io.read_u1()
            self.src_ipv6_addr = self._io.read_bytes(16)
            self.dst_ipv6_addr = self._io.read_bytes(16)
            _on = self.next_header_type
            if _on == 17:
                self.next_header = Lightsail2.UdpDtgrm(self._io, self, self._root)
            elif _on == 0:
                self.next_header = Lightsail2.OptionHopByHop(self._io, self, self._root)
            elif _on == 4:
                self.next_header = Lightsail2.Ipv4Pkt(self._io, self, self._root)
            elif _on == 6:
                self.next_header = Lightsail2.TcpSegm(self._io, self, self._root)
            elif _on == 59:
                self.next_header = Lightsail2.NoNextHeader(self._io, self, self._root)
            self.rest = self._io.read_bytes_full()


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class CamerainfoT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.status = self._io.read_u1()
            self.temp = self._io.read_u1()
            self.last_contact = self._io.read_u1()
            self.pics_remaining = self._io.read_bits_int_be(6)
            self.retry_fails = self._io.read_bits_int_be(2)


    class LsbSysmgrdataT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.type = self._io.read_u1()
            self.daughter_atmp = self._io.read_u1()
            self.daughter_btmp = self._io.read_u1()
            self.threev_pltmp = self._io.read_u1()
            self.rf_amptmp = self._io.read_u1()
            self.nx_tmp = self._io.read_u1()
            self.px_tmp = self._io.read_u1()
            self.ny_tmp = self._io.read_u1()
            self.py_tmp = self._io.read_u1()
            self.nz_tmp = self._io.read_u1()
            self.pz_tmp = self._io.read_u1()
            self.atmelpwrcurr = self._io.read_u1()
            self.atmelpwrbusv = self._io.read_u1()
            self.threev_pwrcurr = self._io.read_u1()
            self.threev_pwrbusv = self._io.read_u1()
            self.threev_plpwrcurr = self._io.read_u1()
            self.threev_plpwrbusv = self._io.read_u1()
            self.fivev_plpwrcurr = self._io.read_u1()
            self.fivev_plpwrbusv = self._io.read_u1()
            self.daughter_apwrcurr = self._io.read_u1()
            self.daughter_apwrbusv = self._io.read_u1()
            self.daughter_bpwrcurr = self._io.read_u1()
            self.daughter_bpwrbusv = self._io.read_u1()
            self.nx_intpwrcurr = self._io.read_u1()
            self.nx_intpwrbusv = self._io.read_u1()
            self.nx_extpwrcurr = self._io.read_u1()
            self.nx_extpwrbusv = self._io.read_u1()
            self.px_intpwrcurr = self._io.read_u1()
            self.px_intpwrbusv = self._io.read_u1()
            self.px_extpwrcurr = self._io.read_u1()
            self.px_extpwrbusv = self._io.read_u1()
            self.ny_intpwrcurr = self._io.read_u1()
            self.ny_intpwrbusv = self._io.read_u1()
            self.ny_extpwrcurr = self._io.read_u1()
            self.ny_extpwrbusv = self._io.read_u1()
            self.py_intpwrcurr = self._io.read_u1()
            self.py_intpwrbusv = self._io.read_u1()
            self.py_extpwrcurr = self._io.read_u1()
            self.py_extpwrbusv = self._io.read_u1()
            self.nz_extpwrcurr = self._io.read_u1()
            self.nz_extpwrbusv = self._io.read_u1()
            self.usercputime = self._io.read_u4be()
            self.syscputime = self._io.read_u4be()
            self.idlecputime = self._io.read_u4be()
            self.processes = self._io.read_u4be()
            self.memfree = self._io.read_u4be()
            self.buffers = self._io.read_u4be()
            self.cached = self._io.read_u4be()
            self.datafree = self._io.read_u4be()
            self.nanderasures = self._io.read_u4be()
            self.beaconcnt = self._io.read_u2be()
            self.time = self._io.read_u4be()
            self.boottime = self._io.read_u4be()
            self.long_dur_counter = self._io.read_u2be()


    class Ipv4Pkt(KaitaiStruct):

        class ProtocolEnum(Enum):
            hopopt = 0
            icmp = 1
            igmp = 2
            ggp = 3
            ipv4 = 4
            st = 5
            tcp = 6
            cbt = 7
            egp = 8
            igp = 9
            bbn_rcc_mon = 10
            nvp_ii = 11
            pup = 12
            argus = 13
            emcon = 14
            xnet = 15
            chaos = 16
            udp = 17
            mux = 18
            dcn_meas = 19
            hmp = 20
            prm = 21
            xns_idp = 22
            trunk_1 = 23
            trunk_2 = 24
            leaf_1 = 25
            leaf_2 = 26
            rdp = 27
            irtp = 28
            iso_tp4 = 29
            netblt = 30
            mfe_nsp = 31
            merit_inp = 32
            dccp = 33
            x_3pc = 34
            idpr = 35
            xtp = 36
            ddp = 37
            idpr_cmtp = 38
            tp_plus_plus = 39
            il = 40
            ipv6 = 41
            sdrp = 42
            ipv6_route = 43
            ipv6_frag = 44
            idrp = 45
            rsvp = 46
            gre = 47
            dsr = 48
            bna = 49
            esp = 50
            ah = 51
            i_nlsp = 52
            swipe = 53
            narp = 54
            mobile = 55
            tlsp = 56
            skip = 57
            ipv6_icmp = 58
            ipv6_nonxt = 59
            ipv6_opts = 60
            any_host_internal_protocol = 61
            cftp = 62
            any_local_network = 63
            sat_expak = 64
            kryptolan = 65
            rvd = 66
            ippc = 67
            any_distributed_file_system = 68
            sat_mon = 69
            visa = 70
            ipcv = 71
            cpnx = 72
            cphb = 73
            wsn = 74
            pvp = 75
            br_sat_mon = 76
            sun_nd = 77
            wb_mon = 78
            wb_expak = 79
            iso_ip = 80
            vmtp = 81
            secure_vmtp = 82
            vines = 83
            ttp = 84
            nsfnet_igp = 85
            dgp = 86
            tcf = 87
            eigrp = 88
            ospfigp = 89
            sprite_rpc = 90
            larp = 91
            mtp = 92
            ax_25 = 93
            ipip = 94
            micp = 95
            scc_sp = 96
            etherip = 97
            encap = 98
            any_private_encryption_scheme = 99
            gmtp = 100
            ifmp = 101
            pnni = 102
            pim = 103
            aris = 104
            scps = 105
            qnx = 106
            a_n = 107
            ipcomp = 108
            snp = 109
            compaq_peer = 110
            ipx_in_ip = 111
            vrrp = 112
            pgm = 113
            any_0_hop = 114
            l2tp = 115
            ddx = 116
            iatp = 117
            stp = 118
            srp = 119
            uti = 120
            smp = 121
            sm = 122
            ptp = 123
            isis_over_ipv4 = 124
            fire = 125
            crtp = 126
            crudp = 127
            sscopmce = 128
            iplt = 129
            sps = 130
            pipe = 131
            sctp = 132
            fc = 133
            rsvp_e2e_ignore = 134
            mobility_header = 135
            udplite = 136
            mpls_in_ip = 137
            manet = 138
            hip = 139
            shim6 = 140
            wesp = 141
            rohc = 142
            reserved_255 = 255
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.b1 = self._io.read_u1()
            self.b2 = self._io.read_u1()
            self.total_length = self._io.read_u2be()
            self.identification = self._io.read_u2be()
            self.b67 = self._io.read_u2be()
            self.ttl = self._io.read_u1()
            self.protocol = KaitaiStream.resolve_enum(Lightsail2.Ipv4Pkt.ProtocolEnum, self._io.read_u1())
            self.header_checksum = self._io.read_u2be()
            self.src_ip_addr = self._io.read_u4be()
            self.dst_ip_addr = self._io.read_u4be()
            self._raw_options = self._io.read_bytes((self.ihl_bytes - 20))
            _io__raw_options = KaitaiStream(BytesIO(self._raw_options))
            self.options = Lightsail2.Ipv4Options(_io__raw_options, self, self._root)
            _on = self.protocol
            if _on == Lightsail2.Ipv4Pkt.ProtocolEnum.udp:
                self._raw_body = self._io.read_bytes((self.total_length - self.ihl_bytes))
                _io__raw_body = KaitaiStream(BytesIO(self._raw_body))
                self.body = Lightsail2.UdpDtgrm(_io__raw_body, self, self._root)
            elif _on == Lightsail2.Ipv4Pkt.ProtocolEnum.icmp:
                self._raw_body = self._io.read_bytes((self.total_length - self.ihl_bytes))
                _io__raw_body = KaitaiStream(BytesIO(self._raw_body))
                self.body = Lightsail2.IcmpPkt(_io__raw_body, self, self._root)
            elif _on == Lightsail2.Ipv4Pkt.ProtocolEnum.ipv6:
                self._raw_body = self._io.read_bytes((self.total_length - self.ihl_bytes))
                _io__raw_body = KaitaiStream(BytesIO(self._raw_body))
                self.body = Lightsail2.Ipv6Pkt(_io__raw_body, self, self._root)
            elif _on == Lightsail2.Ipv4Pkt.ProtocolEnum.tcp:
                self._raw_body = self._io.read_bytes((self.total_length - self.ihl_bytes))
                _io__raw_body = KaitaiStream(BytesIO(self._raw_body))
                self.body = Lightsail2.TcpSegm(_io__raw_body, self, self._root)
            else:
                self.body = self._io.read_bytes((self.total_length - self.ihl_bytes))

        @property
        def version(self):
            if hasattr(self, '_m_version'):
                return self._m_version

            self._m_version = ((self.b1 & 240) >> 4)
            return getattr(self, '_m_version', None)

        @property
        def ihl(self):
            if hasattr(self, '_m_ihl'):
                return self._m_ihl

            self._m_ihl = (self.b1 & 15)
            return getattr(self, '_m_ihl', None)

        @property
        def ihl_bytes(self):
            if hasattr(self, '_m_ihl_bytes'):
                return self._m_ihl_bytes

            self._m_ihl_bytes = (self.ihl * 4)
            return getattr(self, '_m_ihl_bytes', None)


    class TcpSegm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.src_port = self._io.read_u2be()
            self.dst_port = self._io.read_u2be()
            self.seq_num = self._io.read_u4be()
            self.ack_num = self._io.read_u4be()
            self.b12 = self._io.read_u1()
            self.b13 = self._io.read_u1()
            self.window_size = self._io.read_u2be()
            self.checksum = self._io.read_u2be()
            self.urgent_pointer = self._io.read_u2be()
            self.body = self._io.read_bytes_full()


    class NoneL3(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.data = self._io.read_bytes_full()


    class LsbBeacondataT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sys = Lightsail2.LsbSysmgrdataT(self._io, self, self._root)
            self.comm = Lightsail2.LsbCommdataT(self._io, self, self._root)
            self.bat0 = Lightsail2.LsbBatterydataT(self._io, self, self._root)
            self.bat1 = Lightsail2.LsbBatterydataT(self._io, self, self._root)
            self.bat2 = Lightsail2.LsbBatterydataT(self._io, self, self._root)
            self.bat3 = Lightsail2.LsbBatterydataT(self._io, self, self._root)
            self.bat4 = Lightsail2.LsbBatterydataT(self._io, self, self._root)
            self.bat5 = Lightsail2.LsbBatterydataT(self._io, self, self._root)
            self.bat6 = Lightsail2.LsbBatterydataT(self._io, self, self._root)
            self.bat7 = Lightsail2.LsbBatterydataT(self._io, self, self._root)
            self.batt_pwr_draw = self._io.read_s2be()
            self.adcs_mode = self._io.read_u1()
            self.flags = self._io.read_u1()
            self.q0_act = self._io.read_s2be()
            self.q1_act = self._io.read_s2be()
            self.q2_act = self._io.read_s2be()
            self.q3_act = self._io.read_s2be()
            self.x_rate = self._io.read_s2be()
            self.y_rate = self._io.read_s2be()
            self.z_rate = self._io.read_s2be()
            self.gyro_pxy = Lightsail2.Packedsigned2x12T(self._io, self, self._root)
            self.gyro_piz = Lightsail2.Packedsigned2x12T(self._io, self, self._root)
            self.gyro_ixy = Lightsail2.Packedsigned2x12T(self._io, self, self._root)
            self.sol_nxx = self._io.read_u2be()
            self.sol_nxy = self._io.read_u2be()
            self.sol_nyx = self._io.read_u2be()
            self.sol_nyy = self._io.read_u2be()
            self.sol_nzx = self._io.read_u2be()
            self.sol_nzy = self._io.read_u2be()
            self.sol_pxx = self._io.read_u2be()
            self.sol_pxy = self._io.read_u2be()
            self.sol_pyx = self._io.read_u2be()
            self.sol_pyy = self._io.read_u2be()
            self.mag_nxxy = Lightsail2.Packedsigned2x12T(self._io, self, self._root)
            self.mag_npxz = Lightsail2.Packedsigned2x12T(self._io, self, self._root)
            self.mag_pxxy = Lightsail2.Packedsigned2x12T(self._io, self, self._root)
            self.mag_npyz = Lightsail2.Packedsigned2x12T(self._io, self, self._root)
            self.mag_pyxy = Lightsail2.Packedsigned2x12T(self._io, self, self._root)
            self.wheel_rpm = self._io.read_s2be()
            self.cam0 = Lightsail2.CamerainfoT(self._io, self, self._root)
            self.cam1 = Lightsail2.CamerainfoT(self._io, self, self._root)
            self.torqx_pwrcurr = self._io.read_u1()
            self.torqx_pwrbusv = self._io.read_s1()
            self.torqy_pwrcurr = self._io.read_u1()
            self.torqy_pwrbusv = self._io.read_s1()
            self.torqz_pwrcurr = self._io.read_u1()
            self.torqz_pwrbusv = self._io.read_s1()
            self.motor_pwrcurr = self._io.read_u1()
            self.motor_pwrbusv = self._io.read_u1()
            self.pic_panel_flags = self._io.read_u1()
            self.motor_cnt_high = self._io.read_s1()
            self.motor_cnt_low = self._io.read_u2be()
            self.motor_limit_high = self._io.read_s1()
            self.motor_limit_low = self._io.read_u2be()

        @property
        def motor_cnt(self):
            if hasattr(self, '_m_motor_cnt'):
                return self._m_motor_cnt

            self._m_motor_cnt = ((self.motor_cnt_high << 16) | self.motor_cnt_low)
            return getattr(self, '_m_motor_cnt', None)

        @property
        def motor_limit(self):
            if hasattr(self, '_m_motor_limit'):
                return self._m_motor_limit

            self._m_motor_limit = ((self.motor_limit_high << 16) | self.motor_limit_low)
            return getattr(self, '_m_motor_limit', None)


    class UdpDtgrm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.src_port = self._io.read_u2be()
            self.dst_port = self._io.read_u2be()
            self.length = self._io.read_u2be()
            self.checksum = self._io.read_u2be()
            self._raw_body = self._io.read_bytes_full()
            _io__raw_body = KaitaiStream(BytesIO(self._raw_body))
            self.body = Lightsail2.UdpPayload(_io__raw_body, self, self._root)


    class Packedunsigned2x12T(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.a_high = self._io.read_u1()
            self.b_high = self._io.read_u1()
            self.ab_low = self._io.read_u1()

        @property
        def val_a(self):
            if hasattr(self, '_m_val_a'):
                return self._m_val_a

            self._m_val_a = ((self.a_high << 4) | (self.ab_low & 15))
            return getattr(self, '_m_val_a', None)

        @property
        def val_b(self):
            if hasattr(self, '_m_val_b'):
                return self._m_val_b

            self._m_val_b = ((self.b_high << 4) | (self.ab_low & (240 >> 4)))
            return getattr(self, '_m_val_b', None)


    class DestinationUnreachableMsg(KaitaiStruct):

        class DestinationUnreachableCode(Enum):
            net_unreachable = 0
            host_unreachable = 1
            protocol_unreachable = 2
            port_unreachable = 3
            fragmentation_needed_and_df_set = 4
            source_route_failed = 5
            dst_net_unkown = 6
            sdt_host_unkown = 7
            src_isolated = 8
            net_prohibited_by_admin = 9
            host_prohibited_by_admin = 10
            net_unreachable_for_tos = 11
            host_unreachable_for_tos = 12
            communication_prohibited_by_admin = 13
            host_precedence_violation = 14
            precedence_cuttoff_in_effect = 15
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.code = KaitaiStream.resolve_enum(Lightsail2.DestinationUnreachableMsg.DestinationUnreachableCode, self._io.read_u1())
            self.checksum = self._io.read_u2be()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Lightsail2.Callsign(_io__raw_callsign_ror, self, self._root)


    class LsbCommdataT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rxcount = self._io.read_u2be()
            self.txcount = self._io.read_u2be()
            self.rxbytes = self._io.read_u4be()
            self.txbytes = self._io.read_u4be()


    class OptionHopByHop(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.next_header_type = self._io.read_u1()
            self.hdr_ext_len = self._io.read_u1()
            self.body = self._io.read_bytes((self.hdr_ext_len - 1))
            _on = self.next_header_type
            if _on == 0:
                self.next_header = Lightsail2.OptionHopByHop(self._io, self, self._root)
            elif _on == 6:
                self.next_header = Lightsail2.TcpSegm(self._io, self, self._root)
            elif _on == 59:
                self.next_header = Lightsail2.NoNextHeader(self._io, self, self._root)


    class EchoMsg(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.code = self._io.read_bytes(1)
            if not self.code == b"\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\x00", self.code, self._io, u"/types/echo_msg/seq/0")
            self.checksum = self._io.read_u2be()
            self.identifier = self._io.read_u2be()
            self.seq_num = self._io.read_u2be()
            self.data = self._io.read_bytes_full()


    class TimeExceededMsg(KaitaiStruct):

        class TimeExceededCode(Enum):
            time_to_live_exceeded_in_transit = 0
            fragment_reassembly_time_exceeded = 1
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.code = KaitaiStream.resolve_enum(Lightsail2.TimeExceededMsg.TimeExceededCode, self._io.read_u1())
            self.checksum = self._io.read_u2be()


    class IcmpPkt(KaitaiStruct):

        class IcmpTypeEnum(Enum):
            echo_reply = 0
            destination_unreachable = 3
            source_quench = 4
            redirect = 5
            echo = 8
            time_exceeded = 11
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.icmp_type = KaitaiStream.resolve_enum(Lightsail2.IcmpPkt.IcmpTypeEnum, self._io.read_u1())
            if self.icmp_type == Lightsail2.IcmpPkt.IcmpTypeEnum.destination_unreachable:
                self.destination_unreachable = Lightsail2.DestinationUnreachableMsg(self._io, self, self._root)

            if self.icmp_type == Lightsail2.IcmpPkt.IcmpTypeEnum.time_exceeded:
                self.time_exceeded = Lightsail2.TimeExceededMsg(self._io, self, self._root)

            if  ((self.icmp_type == Lightsail2.IcmpPkt.IcmpTypeEnum.echo) or (self.icmp_type == Lightsail2.IcmpPkt.IcmpTypeEnum.echo_reply)) :
                self.echo = Lightsail2.EchoMsg(self._io, self, self._root)



    class Ipv4Option(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.b1 = self._io.read_u1()
            self.len = self._io.read_u1()
            self.body = self._io.read_bytes(((self.len - 2) if self.len > 2 else 0))

        @property
        def copy(self):
            if hasattr(self, '_m_copy'):
                return self._m_copy

            self._m_copy = ((self.b1 & 128) >> 7)
            return getattr(self, '_m_copy', None)

        @property
        def opt_class(self):
            if hasattr(self, '_m_opt_class'):
                return self._m_opt_class

            self._m_opt_class = ((self.b1 & 96) >> 5)
            return getattr(self, '_m_opt_class', None)

        @property
        def number(self):
            if hasattr(self, '_m_number'):
                return self._m_number

            self._m_number = (self.b1 & 31)
            return getattr(self, '_m_number', None)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Bobcat1(KaitaiStruct):
    """:field data: frame.cspheader.data
    :field callsign: frame.data.callsigns.callsign
    :field bobcat1: frame.data.callsigns.bobcat1
    :field bat_v: frame.data.bat_v
    :field bat_i_out: frame.data.bat_i_out
    :field bat_i_in: frame.data.bat_i_in
    :field bootcount_a3200: frame.data.bootcount_a3200
    :field resetcause_a3200: frame.data.resetcause_a3200
    :field bootcause_a3200: frame.data.bootcause_a3200
    :field uptime_a3200: frame.data.uptime_a3200
    :field bootcount_ax100: frame.data.bootcount_ax100
    :field bootcause_ax100: frame.data.bootcause_ax100
    :field i_pwm: frame.data.i_pwm
    :field fs_mounted: frame.data.fs_mounted
    :field antennas_deployed: frame.data.antennas_deployed
    :field deploy_attempts1: frame.data.deploy_attempts1
    :field deploy_attempts2: frame.data.deploy_attempts2
    :field deploy_attempts3: frame.data.deploy_attempts3
    :field deploy_attempts4: frame.data.deploy_attempts4
    :field gyro_x: frame.data.gyro_x
    :field gyro_y: frame.data.gyro_y
    :field gyro_z: frame.data.gyro_z
    :field timestamp: frame.data.timestamp
    :field protocol_version: frame.hk_header.protocol_version
    :field type: frame.hk_header.type
    :field version: frame.hk_header.version
    :field satid: frame.hk_header.satid
    :field checksum: frame.hk_frame.a3200_hktable0.checksum
    :field a3200_hktable0_timestamp: frame.hk_frame.a3200_hktable0.timestamp
    :field source: frame.hk_frame.a3200_hktable0.source
    :field callsign: frame.hk_frame.callsigns.callsign
    :field bobcat1: frame.hk_frame.callsigns.bobcat1
    :field bat_v: frame.hk_frame.bat_v
    :field bat_i_in: frame.hk_frame.bat_i_in
    :field bat_i_out: frame.hk_frame.bat_i_out
    :field solar1_i: frame.hk_frame.solar1_i
    :field solar1_v: frame.hk_frame.solar1_v
    :field solar2_i: frame.hk_frame.solar2_i
    :field solar2_v: frame.hk_frame.solar2_v
    :field solar3_i: frame.hk_frame.solar3_i
    :field solar3_v: frame.hk_frame.solar3_v
    :field novatel_i: frame.hk_frame.novatel_i
    :field sdr_i: frame.hk_frame.sdr_i
    :field bootcount_p31: frame.hk_frame.bootcount_p31
    :field bootcause_p31: frame.hk_frame.bootcause_p31
    :field bootcount_a3200: frame.hk_frame.bootcount_a3200
    :field bootcause_a3200: frame.hk_frame.bootcause_a3200
    :field resetcause_a3200: frame.hk_frame.resetcause_a3200
    :field uptime_a3200: frame.hk_frame.uptime_a3200
    :field temp_mcu: frame.hk_frame.temp_mcu
    :field i_gssb1: frame.hk_frame.i_gssb1
    :field i_pwm: frame.hk_frame.i_pwm
    :field panel_temp1: frame.hk_frame.panel_temp1
    :field panel_temp2: frame.hk_frame.panel_temp2
    :field panel_temp3: frame.hk_frame.panel_temp3
    :field panel_temp4: frame.hk_frame.panel_temp4
    :field panel_temp5: frame.hk_frame.panel_temp5
    :field panel_temp6: frame.hk_frame.panel_temp6
    :field panel_temp7: frame.hk_frame.panel_temp7
    :field panel_temp8: frame.hk_frame.panel_temp8
    :field panel_temp9: frame.hk_frame.panel_temp9
    :field p31_temp1: frame.hk_frame.p31_temp1
    :field p31_temp2: frame.hk_frame.p31_temp2
    :field p31_temp3: frame.hk_frame.p31_temp3
    :field p31_temp4: frame.hk_frame.p31_temp4
    :field p31_temp5: frame.hk_frame.p31_temp5
    :field p31_temp6: frame.hk_frame.p31_temp6
    :field flash0_free: frame.hk_frame.flash0_free
    :field flash1_free: frame.hk_frame.flash1_free
    :field coll_running: frame.hk_frame.coll_running
    :field ax100_telemtable_checksum: frame.hk_frame.ax100_telemtable.checksum
    :field ax100_telemtable_timestamp: frame.hk_frame.ax100_telemtable.timestamp
    :field ax100_telemtable_source: frame.hk_frame.ax100_telemtable.source
    :field temp_brd: frame.hk_frame.temp_brd
    :field temp_pa: frame.hk_frame.temp_pa
    :field bgnd_rssi: frame.hk_frame.bgnd_rssi
    :field tot_tx_count: frame.hk_frame.tot_tx_count
    :field tot_rx_count: frame.hk_frame.tot_rx_count
    :field tot_tx_bytes: frame.hk_frame.tot_tx_bytes
    :field tot_rx_bytes: frame.hk_frame.tot_rx_bytes
    :field bootcount_ax100: frame.hk_frame.bootcount_ax100
    :field bootcause_ax100: frame.hk_frame.bootcause_ax100
    :field a3200_hktable_17_1_checksum: frame.extra_frame.a3200_hktable_17_1.checksum
    :field a3200_hktable_17_1_timestamp: frame.extra_frame.a3200_hktable_17_1.timestamp
    :field a3200_hktable_17_1_source: frame.extra_frame.a3200_hktable_17_1.source
    :field callsign: frame.extra_frame.callsigns.callsign
    :field bobcat1: frame.extra_frame.callsigns.bobcat1
    :field coll_running: frame.extra_frame.coll_running
    :field lat: frame.extra_frame.lat
    :field long: frame.extra_frame.long
    :field height: frame.extra_frame.height
    :field sec_of_week: frame.extra_frame.sec_of_week
    :field a3200_hktable_10_checksum: frame.extra_frame.a3200_hktable_10.checksum
    :field a3200_hktable_10_timestamp: frame.extra_frame.a3200_hktable_10.timestamp
    :field a3200_hktable_10_source: frame.extra_frame.a3200_hktable_10.source
    :field cfg_path: frame.extra_frame.cfg_path
    :field bc1_wdcnt: frame.extra_frame.bc1_wdcnt
    :field collect_id: frame.extra_frame.collect_id
    :field adcs_mode: frame.extra_frame.adcs_mode
    :field a3200_hktable_17_2_checksum: frame.extra_frame.a3200_hktable_17_2.checksum
    :field a3200_hktable_17_2_timestamp: frame.extra_frame.a3200_hktable_17_2.timestamp
    :field a3200_hktable_17_2_source: frame.extra_frame.a3200_hktable_17_2.source
    :field custom_message: frame.extra_frame.custom_message
    :field rev: frame.fwdebug_frame.rev
    :field swload_cnt1: frame.fwdebug_frame.swload_cnt1
    :field swload_cnt2: frame.fwdebug_frame.swload_cnt2
    :field ram_image: frame.fwdebug_frame.ram_image
    :field framelength: framelength
    :field pos_sol_status: frame.xyz_frame.pos_sol_status
    :field gps_week: frame.xyz_frame.gps_week
    :field gps_ms: frame.xyz_frame.gps_ms
    :field pos_x: frame.xyz_frame.pos_x
    :field pos_y: frame.xyz_frame.pos_y
    :field pos_z: frame.xyz_frame.pos_z
    :field vel_x: frame.xyz_frame.vel_x
    :field vel_y: frame.xyz_frame.vel_y
    :field vel_z: frame.xyz_frame.vel_z
    :field vel_latency: frame.xyz_frame.vel_latency
    :field diff_age: frame.xyz_frame.diff_age
    :field sol_age: frame.xyz_frame.sol_age
    :field num_soln_svs: frame.xyz_frame.num_soln_svs
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        _on = self._root.framelength
        if _on == 150:
            self.frame = Bobcat1.Bc1ExtraFrame(self._io, self, self._root)
        elif _on == 98:
            self.frame = Bobcat1.Bc1XyzFrame(self._io, self, self._root)
        elif _on == 69:
            self.frame = Bobcat1.Bc1BasicFrame(self._io, self, self._root)
        elif _on == 156:
            self.frame = Bobcat1.Bc1HkFrame(self._io, self, self._root)
        elif _on == 70:
            self.frame = Bobcat1.Bc1FwdebugFrame(self._io, self, self._root)

    class BeaconElementHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.checksum = self._io.read_u2be()
            self.timestamp = self._io.read_u4be()
            self.source = self._io.read_u2be()


    class XyzFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsigns = Bobcat1.Callsigns(self._io, self, self._root)
            self.pos_sol_status = self._io.read_u4be()
            self.gps_week = self._io.read_u2be()
            self.gps_ms = self._io.read_u4be()
            self.pos_x = self._io.read_f8be()
            self.pos_y = self._io.read_f8be()
            self.pos_z = self._io.read_f8be()
            self.vel_x = self._io.read_f8be()
            self.vel_y = self._io.read_f8be()
            self.vel_z = self._io.read_f8be()
            self.vel_latency = self._io.read_f4be()
            self.diff_age = self._io.read_f4be()
            self.sol_age = self._io.read_f4be()
            self.num_soln_svs = self._io.read_u1()


    class Cspheader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.data = self._io.read_u4be()


    class Bc1XyzFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.cspheader = Bobcat1.Cspheader(self._io, self, self._root)
            self.xyz_frame = Bobcat1.XyzFrame(self._io, self, self._root)


    class Bc1HkFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.cspheader = Bobcat1.Cspheader(self._io, self, self._root)
            self.hk_header = Bobcat1.HkHeader(self._io, self, self._root)
            self.hk_frame = Bobcat1.HkData(self._io, self, self._root)


    class ExtraData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.a3200_hktable_17_1 = Bobcat1.BeaconElementHeader(self._io, self, self._root)
            self.callsigns = Bobcat1.Callsigns(self._io, self, self._root)
            self.coll_running = self._io.read_u1()
            self.lat = self._io.read_f4be()
            self.long = self._io.read_f4be()
            self.height = self._io.read_f4be()
            self.sec_of_week = self._io.read_u2be()
            self.a3200_hktable_10 = Bobcat1.BeaconElementHeader(self._io, self, self._root)
            self.cfg_path = (KaitaiStream.bytes_terminate(self._io.read_bytes(50), 0, False)).decode(u"ASCII")
            self.bc1_wdcnt = self._io.read_u4be()
            self.collect_id = self._io.read_u4be()
            self.adcs_mode = self._io.read_u1()
            self.a3200_hktable_17_2 = Bobcat1.BeaconElementHeader(self._io, self, self._root)
            self.custom_message = (KaitaiStream.bytes_terminate(self._io.read_bytes(20), 0, False)).decode(u"ascii")


    class HkData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.a3200_hktable0 = Bobcat1.BeaconElementHeader(self._io, self, self._root)
            self.callsigns = Bobcat1.Callsigns(self._io, self, self._root)
            self.bat_v = self._io.read_u2be()
            self.bat_i_in = self._io.read_u2be()
            self.bat_i_out = self._io.read_u2be()
            self.solar1_i = self._io.read_u2be()
            self.solar1_v = self._io.read_u2be()
            self.solar2_i = self._io.read_u2be()
            self.solar2_v = self._io.read_u2be()
            self.solar3_i = self._io.read_u2be()
            self.solar3_v = self._io.read_u2be()
            self.novatel_i = self._io.read_u2be()
            self.sdr_i = self._io.read_u2be()
            self.bootcount_p31 = self._io.read_u4be()
            self.bootcause_p31 = self._io.read_u1()
            self.bootcount_a3200 = self._io.read_u2be()
            self.bootcause_a3200 = self._io.read_u1()
            self.resetcause_a3200 = self._io.read_u1()
            self.uptime_a3200 = self._io.read_u4be()
            self.temp_mcu = self._io.read_s2be()
            self.i_gssb1 = self._io.read_u2be()
            self.i_pwm = self._io.read_u2be()
            self.panel_temp1 = self._io.read_s2be()
            self.panel_temp2 = self._io.read_s2be()
            self.panel_temp3 = self._io.read_s2be()
            self.panel_temp4 = self._io.read_s2be()
            self.panel_temp5 = self._io.read_s2be()
            self.panel_temp6 = self._io.read_s2be()
            self.panel_temp7 = self._io.read_s2be()
            self.panel_temp8 = self._io.read_s2be()
            self.panel_temp9 = self._io.read_s2be()
            self.p31_temp1 = self._io.read_s2be()
            self.p31_temp2 = self._io.read_s2be()
            self.p31_temp3 = self._io.read_s2be()
            self.p31_temp4 = self._io.read_s2be()
            self.p31_temp5 = self._io.read_s2be()
            self.p31_temp6 = self._io.read_s2be()
            self.flash0_free = self._io.read_u4be()
            self.flash1_free = self._io.read_u4be()
            self.coll_running = self._io.read_u1()
            self.ax100_telemtable = Bobcat1.BeaconElementHeader(self._io, self, self._root)
            self.temp_brd = self._io.read_s2be()
            self.temp_pa = self._io.read_s2be()
            self.bgnd_rssi = self._io.read_s2be()
            self.tot_tx_count = self._io.read_u4be()
            self.tot_rx_count = self._io.read_u4be()
            self.tot_tx_bytes = self._io.read_u4be()
            self.tot_rx_bytes = self._io.read_u4be()
            self.bootcount_ax100 = self._io.read_u2be()
            self.bootcause_ax100 = self._io.read_u4be()


    class FwdebugFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.a3200_hktable17 = Bobcat1.BeaconElementHeader(self._io, self, self._root)
            self.callsigns = Bobcat1.Callsigns(self._io, self, self._root)
            self.a3200_hktable0 = Bobcat1.BeaconElementHeader(self._io, self, self._root)
            self.rev = self._io.read_u1()
            self.a3200_hktable1 = Bobcat1.BeaconElementHeader(self._io, self, self._root)
            self.swload_cnt1 = self._io.read_u2be()
            self.swload_cnt2 = self._io.read_u2be()
            self.a3200_hktable4 = Bobcat1.BeaconElementHeader(self._io, self, self._root)
            self.ram_image = self._io.read_u1()


    class Bc1ExtraFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.cspheader = Bobcat1.Cspheader(self._io, self, self._root)
            self.hk_header = Bobcat1.HkHeader(self._io, self, self._root)
            self.extra_frame = Bobcat1.ExtraData(self._io, self, self._root)


    class HkHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.protocol_version = self._io.read_u1()
            self.type = self._io.read_u1()
            self.version = self._io.read_u1()
            self.satid = self._io.read_u2be()


    class Basic(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsigns = Bobcat1.Callsigns(self._io, self, self._root)
            self.bat_v = self._io.read_u2be()
            self.bat_i_out = self._io.read_u2be()
            self.bat_i_in = self._io.read_u2be()
            self.bootcount_a3200 = self._io.read_u2be()
            self.resetcause_a3200 = self._io.read_u1()
            self.bootcause_a3200 = self._io.read_u1()
            self.uptime_a3200 = self._io.read_u4be()
            self.bootcount_ax100 = self._io.read_u2be()
            self.bootcause_ax100 = self._io.read_u4be()
            self.i_pwm = self._io.read_u2be()
            self.fs_mounted = self._io.read_u1()
            self.antennas_deployed = self._io.read_u1()
            self.deploy_attempts1 = self._io.read_u2be()
            self.deploy_attempts2 = self._io.read_u2be()
            self.deploy_attempts3 = self._io.read_u2be()
            self.deploy_attempts4 = self._io.read_u2be()
            self.gyro_x = self._io.read_s2be()
            self.gyro_y = self._io.read_s2be()
            self.gyro_z = self._io.read_s2be()
            self.timestamp = self._io.read_u4be()


    class Bc1FwdebugFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.cspheader = Bobcat1.Cspheader(self._io, self, self._root)
            self.hk_header = Bobcat1.HkHeader(self._io, self, self._root)
            self.fwdebug_frame = Bobcat1.FwdebugFrame(self._io, self, self._root)


    class Callsigns(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes_term(0, False, True, True)).decode(u"ASCII")
            if not self.callsign == u"W8PZS":
                raise kaitaistruct.ValidationNotEqualError(u"W8PZS", self.callsign, self._io, u"/types/callsigns/seq/0")
            self.bobcat1 = (self._io.read_bytes_term(0, False, True, True)).decode(u"ASCII")
            if not self.bobcat1 == u"BOBCAT-1":
                raise kaitaistruct.ValidationNotEqualError(u"BOBCAT-1", self.bobcat1, self._io, u"/types/callsigns/seq/1")


    class Bc1BasicFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.cspheader = Bobcat1.Cspheader(self._io, self, self._root)
            self.data = Bobcat1.Basic(self._io, self, self._root)


    @property
    def framelength(self):
        if hasattr(self, '_m_framelength'):
            return self._m_framelength

        self._m_framelength = self._io.size()
        return getattr(self, '_m_framelength', None)





# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Entrysat(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_callsign: ax25_frame.ax25_header.repeater.rpt_instance[0].rpt_callsign_raw.callsign_ror.callsign
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field packet_id: ax25_frame.payload.ax25_info.pus_packet.tm_packet_header.packet_id
    :field packet_seq_ctl: ax25_frame.payload.ax25_info.pus_packet.tm_packet_header.packet_seq_ctl
    :field packet_length: ax25_frame.payload.ax25_info.pus_packet.tm_packet_header.packet_length
    :field service: ax25_frame.payload.ax25_info.pus_packet.tm_packet_header.service
    :field subservice: ax25_frame.payload.ax25_info.pus_packet.tm_packet_header.subservice
    :field clock: ax25_frame.payload.ax25_info.pus_packet.tm_packet_header.clock
    :field sid: ax25_frame.payload.ax25_info.pus_packet.payload.sid
    :field mode: ax25_frame.payload.ax25_info.pus_packet.payload.tm_payload.mode
    :field eps_vbatt: ax25_frame.payload.ax25_info.pus_packet.payload.tm_payload.eps_vbatt
    :field eps_batt_vcurrent: ax25_frame.payload.ax25_info.pus_packet.payload.tm_payload.eps_batt_vcurrent
    :field eps_3v3_current: ax25_frame.payload.ax25_info.pus_packet.payload.tm_payload.eps_3v3_current
    :field eps_5v_current: ax25_frame.payload.ax25_info.pus_packet.payload.tm_payload.eps_5v_current
    :field trx_temp: ax25_frame.payload.ax25_info.pus_packet.payload.tm_payload.trx_temp
    :field eps_temp: ax25_frame.payload.ax25_info.pus_packet.payload.tm_payload.eps_temp
    :field batt_temp: ax25_frame.payload.ax25_info.pus_packet.payload.tm_payload.batt_temp
    :field frame_status: ax25_frame.payload.ax25_info.pus_packet.payload.tm_payload.frame_status
    :field timestamp: ax25_frame.payload.ax25_info.pus_packet.payload.tm_payload.timestamp
    
    
    Attention: `rpt_callsign` cannot be accessed because `rpt_instance` is an
    array of unknown size at the beginning of the parsing process! Left an
    example in here.
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Entrysat.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Entrysat.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Entrysat.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Entrysat.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Entrysat.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Entrysat.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Entrysat.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Entrysat.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Entrysat.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Entrysat.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Entrysat.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Entrysat.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.repeater = Entrysat.Repeater(self._io, self, self._root)

            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Entrysat.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Entrysat.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class PusPacketT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp_unused = self._io.read_u4be()
            self.tm_packet_header = Entrysat.TmPacketHeaderT(self._io, self, self._root)
            self.payload = Entrysat.TmPacketT(self._io, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Entrysat.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Entrysat.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Entrysat.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class TmPacketHeaderT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.packet_id = self._io.read_u2be()
            self.packet_seq_ctl = self._io.read_u2be()
            self.packet_length = self._io.read_u2be()
            self.spare_tm = self._io.read_u1()
            self.service = self._io.read_u1()
            self.subservice = self._io.read_u1()
            self.clock_array = []
            for i in range(5):
                self.clock_array.append(self._io.read_u1())


        @property
        def clock(self):
            if hasattr(self, '_m_clock'):
                return self._m_clock

            self._m_clock = (((((((self.clock_array[0] * 256) * 256) * 256) + ((self.clock_array[1] * 256) * 256)) + (self.clock_array[2] * 256)) + self.clock_array[3]) + (self.clock_array[4] / 256.0))
            return getattr(self, '_m_clock', None)


    class TmPacketT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sid = self._io.read_u1()
            _on = self.sid
            if _on == 6:
                self.tm_payload = Entrysat.EpsPacketT(self._io, self, self._root)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Entrysat.Callsign(_io__raw_callsign_ror, self, self._root)


    class EpsPacketT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.mode = self._io.read_u1()
            self.eps_vbatt = self._io.read_u1()
            self.eps_batt_vcurrent = self._io.read_u1()
            self.eps_3v3_current = self._io.read_u1()
            self.eps_5v_current = self._io.read_u1()
            self.trx_temp = self._io.read_u1()
            self.eps_temp = self._io.read_u1()
            self.batt_temp = self._io.read_u1()
            self.crc = self._io.read_u2be()
            self.frame_status = self._io.read_u1()
            self.timestamp = self._io.read_u4le()


    class Ax25InfoData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pus_packet = Entrysat.PusPacketT(self._io, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
from enum import Enum


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Kafasat1(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field version: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.version
    :field type: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.type
    :field sec_hdr_flag: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.sec_hdr_flag
    :field pkt_apid: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.pkt_apid
    :field seq_flgs: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.seq_flgs
    :field seq_ctr: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.seq_ctr
    :field pkt_len: ax25_frame.payload.ax25_info.ccsds_space_packet.packet_primary_header.pkt_len
    :field pus_header: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.secondary_header.pus_header
    :field service_type: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.secondary_header.service_type
    :field service_subtype: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.secondary_header.service_subtype
    :field sys_mode: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.mode.sys_mode
    :field battery_mode: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.mode.battery_mode
    :field vbat: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.battery.vbat
    :field cbat: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.battery.cbat
    :field cbchg: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.battery.cbchg
    :field st_5v: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.battery.st_5v
    :field temperature_bat1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.battery.temperature_bat1
    :field temperature_bat2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.battery.temperature_bat2
    :field heater: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.battery.heater
    :field gnd_wdt: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.gnd_wdt
    :field vin0: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.acu.vin0
    :field vin1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.acu.vin1
    :field vin2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.acu.vin2
    :field vin3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.acu.vin3
    :field vin4: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.acu.vin4
    :field vin5: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.acu.vin5
    :field cin0: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.acu.cin0
    :field cin1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.acu.cin1
    :field cin2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.acu.cin2
    :field cin3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.acu.cin3
    :field cin4: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.acu.cin4
    :field cin5: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.acu.cin5
    :field sw_cam_vout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_cam_vout
    :field sw_hrm1_vout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_hrm1_vout
    :field sw_ant_vout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_ant_vout
    :field sw_wheel_vout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_wheel_vout
    :field sw_trxvu_vout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_trxvu_vout
    :field sw_sband_vout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_sband_vout
    :field sw_hrm2_vout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_hrm2_vout
    :field sw_mtqr_vout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_mtqr_vout
    :field sw_adcs_vout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_adcs_vout
    :field sw_cam_cout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_cam_cout
    :field sw_hrm1_cout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_hrm1_cout
    :field sw_ant_cout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_ant_cout
    :field sw_wheel_cout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_wheel_cout
    :field sw_trxvu_cout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_trxvu_cout
    :field sw_sband_cout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_sband_cout
    :field sw_hrm2_cout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_hrm2_cout
    :field sw_mtqr_cout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_mtqr_cout
    :field sw_adcs_cout: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.pdu.sw_adcs_cout
    :field hrm1_deploy: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.deploy.hrm1_deploy
    :field hrm2_deploy: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.deploy.hrm2_deploy
    :field hrm1_deploy_count: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.deploy.hrm1_deploy_count
    :field hrm2_deploy_count: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.deploy.hrm2_deploy_count
    :field rx_uptime: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.uptime.rx_uptime
    :field tx_uptime: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.uptime.tx_uptime
    :field attitude_estimation_mode: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.attitude_estimation_mode
    :field control_mode: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.control_mode
    :field adcs_run_mode: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.adcs_run_mode
    :field asgp4_mode: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.asgp4_mode
    :field cubecontrol_signal_enabled: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.cubecontrol_signal_enabled
    :field cubecontrol_motor_enabled: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.cubecontrol_motor_enabled
    :field cube_sense_enabled: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.cube_sense_enabled
    :field cube_enabled1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.cube_enabled1
    :field cube_error1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.cube_error1
    :field cube_error2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.cube_error2
    :field cube_error3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.cube_error3
    :field estimated_roll_angle: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.estimated_roll_angle
    :field estimated_pitch_angle: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.estimated_pitch_angle
    :field estimated_yaw_angle: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.estimated_yaw_angle
    :field estimated_q1: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.estimated_q1
    :field estimated_q2: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.estimated_q2
    :field estimated_q3: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.estimated_q3
    :field estimated_x_angular_rate: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.estimated_x_angular_rate
    :field estimated_y_angular_rate: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.estimated_y_angular_rate
    :field estimated_z_angular_rate: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.estimated_z_angular_rate
    :field position_x: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.position_x
    :field position_y: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.position_y
    :field position_z: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.position_z
    :field velocity_x: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.velocity_x
    :field velocity_y: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.velocity_y
    :field velocity_z: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.velocity_z
    :field latitude: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.latitude
    :field longitude: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.longitude
    :field altitude: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.altitude
    :field ecef_x: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.ecef_x
    :field ecef_y: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.ecef_y
    :field ecef_z: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_state.ecef_z
    :field magnetic_x: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.magnetic_x
    :field magnetic_y: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.magnetic_y
    :field magnetic_z: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.magnetic_z
    :field css_x: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.css_x
    :field css_y: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.css_y
    :field css_z: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.css_z
    :field sun_x: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.sun_x
    :field sun_x: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.sun_y
    :field sun_x: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.sun_z
    :field nadir_x: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.nadir_x
    :field nadir_y: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.nadir_y
    :field nadir_z: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.nadir_z
    :field angular_rate_x: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.angular_rate_x
    :field angular_rate_y: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.angular_rate_y
    :field angular_rate_z: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.angular_rate_z
    :field wheel_x: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.wheel_x
    :field wheel_y: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.wheel_y
    :field wheel_z: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_measure.wheel_z
    :field adcs_seconds: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_seconds
    :field adcs_subseconds: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.adcs_subseconds
    :field boot_count: ax25_frame.payload.ax25_info.ccsds_space_packet.data_section.user_data_field.boot_count
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Kafasat1.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Kafasat1.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Kafasat1.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Kafasat1.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Kafasat1.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Kafasat1.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Kafasat1.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Kafasat1.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Kafasat1.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Kafasat1.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Kafasat1.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Kafasat1.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class AdcsMeasureT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.magnetic_x = self._io.read_s2le()
            self.magnetic_y = self._io.read_s2le()
            self.magnetic_z = self._io.read_s2le()
            self.css_x = self._io.read_s2le()
            self.css_y = self._io.read_s2le()
            self.css_z = self._io.read_s2le()
            self.sun_x = self._io.read_s2le()
            self.sun_y = self._io.read_s2le()
            self.sun_z = self._io.read_s2le()
            self.nadir_x = self._io.read_s2le()
            self.nadir_y = self._io.read_s2le()
            self.nadir_z = self._io.read_s2le()
            self.angular_rate_x = self._io.read_s2le()
            self.angular_rate_y = self._io.read_s2le()
            self.angular_rate_z = self._io.read_s2le()
            self.wheel_x = self._io.read_s2le()
            self.wheel_y = self._io.read_s2le()
            self.wheel_z = self._io.read_s2le()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Kafasat1.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class AcuT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.vin0 = self._io.read_u2le()
            self.vin1 = self._io.read_u2le()
            self.vin2 = self._io.read_u2le()
            self.vin3 = self._io.read_u2le()
            self.vin4 = self._io.read_u2le()
            self.vin5 = self._io.read_u2le()
            self.cin0 = self._io.read_s2le()
            self.cin1 = self._io.read_s2le()
            self.cin2 = self._io.read_s2le()
            self.cin3 = self._io.read_s2le()
            self.cin4 = self._io.read_s2le()
            self.cin5 = self._io.read_s2le()


    class DeployT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.raw_deploy_stat = []
            for i in range(8):
                self.raw_deploy_stat.append(self._io.read_bits_int_be(1) != 0)

            self._io.align_to_byte()
            self.hrm1_deploy_count = self._io.read_u2le()
            self.hrm2_deploy_count = self._io.read_u2le()

        @property
        def hrm1_deploy(self):
            if hasattr(self, '_m_hrm1_deploy'):
                return self._m_hrm1_deploy

            self._m_hrm1_deploy = (True if self.raw_deploy_stat[0] else False)
            return getattr(self, '_m_hrm1_deploy', None)

        @property
        def hrm2_deploy(self):
            if hasattr(self, '_m_hrm2_deploy'):
                return self._m_hrm2_deploy

            self._m_hrm2_deploy = (True if self.raw_deploy_stat[2] else False)
            return getattr(self, '_m_hrm2_deploy', None)


    class AdcsStateT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.attitude_estimation_mode = self._io.read_bits_int_be(4)
            self.control_mode = self._io.read_bits_int_be(4)
            self.adcs_run_mode = self._io.read_bits_int_be(2)
            self.asgp4_mode = self._io.read_bits_int_be(2)
            self.cubecontrol_signal_enabled = self._io.read_bits_int_be(1) != 0
            self.cubecontrol_motor_enabled = self._io.read_bits_int_be(1) != 0
            self.cube_sense_enabled = self._io.read_bits_int_be(2)
            self._io.align_to_byte()
            self.cube_enabled1 = self._io.read_u1()
            self.cube_error1 = self._io.read_u1()
            self.cube_error2 = self._io.read_u4le()
            self.cube_error3 = self._io.read_u4le()
            self.estimated_roll_angle = self._io.read_s2le()
            self.estimated_pitch_angle = self._io.read_s2le()
            self.estimated_yaw_angle = self._io.read_s2le()
            self.estimated_q1 = self._io.read_s2le()
            self.estimated_q2 = self._io.read_s2le()
            self.estimated_q3 = self._io.read_s2le()
            self.estimated_x_angular_rate = self._io.read_s2le()
            self.estimated_y_angular_rate = self._io.read_s2le()
            self.estimated_z_angular_rate = self._io.read_s2le()
            self.position_x = self._io.read_s2le()
            self.position_y = self._io.read_s2le()
            self.position_z = self._io.read_s2le()
            self.velocity_x = self._io.read_s2le()
            self.velocity_y = self._io.read_s2le()
            self.velocity_z = self._io.read_s2le()
            self.latitude = self._io.read_s2le()
            self.longitude = self._io.read_s2le()
            self.altitude = self._io.read_u2le()
            self.ecef_x = self._io.read_s2le()
            self.ecef_y = self._io.read_s2le()
            self.ecef_z = self._io.read_s2le()


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Kafasat1.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class DataSectionT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw_secondary_header = self._io.read_bytes(3)
            _io__raw_secondary_header = KaitaiStream(BytesIO(self._raw_secondary_header))
            self.secondary_header = Kafasat1.SecondaryHeaderT(_io__raw_secondary_header, self, self._root)
            self.user_data_field = Kafasat1.KafasatDataFieldT(self._io, self, self._root)


    class KafasatDataFieldT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw_mode = self._io.read_bytes(2)
            _io__raw_mode = KaitaiStream(BytesIO(self._raw_mode))
            self.mode = Kafasat1.ModeT(_io__raw_mode, self, self._root)
            self._raw_battery = self._io.read_bytes(13)
            _io__raw_battery = KaitaiStream(BytesIO(self._raw_battery))
            self.battery = Kafasat1.BatteryT(_io__raw_battery, self, self._root)
            self.gnd_wdt = self._io.read_u4le()
            self._raw_acu = self._io.read_bytes(24)
            _io__raw_acu = KaitaiStream(BytesIO(self._raw_acu))
            self.acu = Kafasat1.AcuT(_io__raw_acu, self, self._root)
            self._raw_pdu = self._io.read_bytes(36)
            _io__raw_pdu = KaitaiStream(BytesIO(self._raw_pdu))
            self.pdu = Kafasat1.PduT(_io__raw_pdu, self, self._root)
            self._raw_deploy = self._io.read_bytes(5)
            _io__raw_deploy = KaitaiStream(BytesIO(self._raw_deploy))
            self.deploy = Kafasat1.DeployT(_io__raw_deploy, self, self._root)
            self._raw_uptime = self._io.read_bytes(8)
            _io__raw_uptime = KaitaiStream(BytesIO(self._raw_uptime))
            self.uptime = Kafasat1.UptimeT(_io__raw_uptime, self, self._root)
            self._raw_adcs_state = self._io.read_bytes(54)
            _io__raw_adcs_state = KaitaiStream(BytesIO(self._raw_adcs_state))
            self.adcs_state = Kafasat1.AdcsStateT(_io__raw_adcs_state, self, self._root)
            self._raw_adcs_measure = self._io.read_bytes(36)
            _io__raw_adcs_measure = KaitaiStream(BytesIO(self._raw_adcs_measure))
            self.adcs_measure = Kafasat1.AdcsMeasureT(_io__raw_adcs_measure, self, self._root)
            self.adcs_seconds = self._io.read_u4le()
            self.adcs_subseconds = self._io.read_u2le()
            self.boot_count = self._io.read_u2le()


    class UptimeT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rx_uptime = self._io.read_u4le()
            self.tx_uptime = self._io.read_u4le()


    class SecondaryHeaderT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pus_header = self._io.read_u1()
            self.service_type = self._io.read_u1()
            self.service_subtype = self._io.read_u1()


    class PduT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sw_cam_vout = self._io.read_u2le()
            self.sw_hrm1_vout = self._io.read_u2le()
            self.sw_ant_vout = self._io.read_u2le()
            self.sw_wheel_vout = self._io.read_u2le()
            self.sw_trxvu_vout = self._io.read_u2le()
            self.sw_sband_vout = self._io.read_u2le()
            self.sw_hrm2_vout = self._io.read_u2le()
            self.sw_mtqr_vout = self._io.read_u2le()
            self.sw_adcs_vout = self._io.read_u2le()
            self.sw_cam_cout = self._io.read_s2le()
            self.sw_hrm1_cout = self._io.read_s2le()
            self.sw_ant_cout = self._io.read_s2le()
            self.sw_wheel_cout = self._io.read_s2le()
            self.sw_trxvu_cout = self._io.read_s2le()
            self.sw_sband_cout = self._io.read_s2le()
            self.sw_hrm2_cout = self._io.read_s2le()
            self.sw_mtqr_cout = self._io.read_s2le()
            self.sw_adcs_cout = self._io.read_s2le()


    class PacketPrimaryHeaderT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.version = self._io.read_bits_int_be(3)
            self.type = self._io.read_bits_int_be(1) != 0
            self.sec_hdr_flag = self._io.read_bits_int_be(1) != 0
            self.pkt_apid = self._io.read_bits_int_be(11)
            self.seq_flgs = self._io.read_bits_int_be(2)
            self.seq_ctr = self._io.read_bits_int_be(14)
            self._io.align_to_byte()
            self.pkt_len = self._io.read_u2be()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Kafasat1.Callsign(_io__raw_callsign_ror, self, self._root)


    class CcsdsSpacePacketT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw_packet_primary_header = self._io.read_bytes(6)
            _io__raw_packet_primary_header = KaitaiStream(BytesIO(self._raw_packet_primary_header))
            self.packet_primary_header = Kafasat1.PacketPrimaryHeaderT(_io__raw_packet_primary_header, self, self._root)
            self.data_section = Kafasat1.DataSectionT(self._io, self, self._root)


    class ModeT(KaitaiStruct):

        class ObcMode(Enum):
            normal = 2
            safe = 6

        class P60dockMode(Enum):
            critical = 1
            safe = 2
            normal = 3
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sys_mode = KaitaiStream.resolve_enum(Kafasat1.ModeT.ObcMode, self._io.read_u1())
            self.battery_mode = KaitaiStream.resolve_enum(Kafasat1.ModeT.P60dockMode, self._io.read_u1())


    class BatteryT(KaitaiStruct):

        class HeaterState(Enum):
            false = 0
            true = 1
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.vbat = self._io.read_u2le()
            self.cbat = self._io.read_s2le()
            self.cbchg = self._io.read_s2le()
            self.st_5v = self._io.read_u2le()
            self.tbat1 = self._io.read_s2le()
            self.tbat2 = self._io.read_s2le()
            self.heater = KaitaiStream.resolve_enum(Kafasat1.BatteryT.HeaterState, self._io.read_u1())

        @property
        def temperature_bat1(self):
            if hasattr(self, '_m_temperature_bat1'):
                return self._m_temperature_bat1

            self._m_temperature_bat1 = (self.tbat1 * 0.1)
            return getattr(self, '_m_temperature_bat1', None)

        @property
        def temperature_bat2(self):
            if hasattr(self, '_m_temperature_bat2'):
                return self._m_temperature_bat2

            self._m_temperature_bat2 = (self.tbat2 * 0.1)
            return getattr(self, '_m_temperature_bat2', None)


    class Ax25InfoData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ccsds_space_packet = Kafasat1.CcsdsSpacePacketT(self._io, self, self._root)



